/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.DocumentNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementEdge;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class ReferenceGraphDataBuilder extends DefaultHandler implements IDataBuilder
{
	/**
	 * The ID of this data builder
	 */
	public static final String ID = SMLActivator.PLUGIN_ID + ".ReferenceGraphDataBuilder";

	/**
	 * Represents the reference graph
	 * 
	 * KEY = The alias representing a document node
	 * VALUE = An object of type {@link DocumentNode}
	 */
	private Map referenceGraph;
	
	/**
	 * The current node which represents a document 
	 */
	private DocumentNode currentNode;
	
	/**
	 * Set to true when notified of the start of an alias element
	 */
	private boolean aliasElementHit;
	
	/**
	 * Set to true when notified of the start of an uri element
	 */
	private boolean uriElementHit;
	
	/**
	 * Stores the source node of a reference
	 */
	private String srcReferenceNode;
	
	/**
	 * Stores the last element that has been hit
	 */
	private String lastElement;
			
	/**
	 * Current edge of the reference graph node
	 */
	private ElementEdge elementEdge;
	
	/**
	 * The current alias
	 */
	private String currentAlias;
	
	
	/**
	 * Constructor
	 */
	public ReferenceGraphDataBuilder()
	{
		referenceGraph = new Hashtable();
		currentAlias = "";
	}
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		resetElementStartStatus();
		if (ISMLConstants.DOCUMENT_ELEMENT.equals(qName))
		{							
			currentNode = new DocumentNode();					
		}
		else if (ISMLConstants.ALIAS_ELEMENT.equals(qName))
		{
			aliasElementHit = true;
		}
		else if (ISMLConstants.URI_ELEMENT.equals(qName))
		{
			uriElementHit = true;
			srcReferenceNode = lastElement;
		}
		
		lastElement = qName;
	}

	/**
	 * Resets flags that are set when the handler is notified of the start 
	 * of a specific element. 
	 */
	private void resetElementStartStatus()
	{
		aliasElementHit = false;
		elementEdge = null;
	}

	/**
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		String textNode = new String(ch, start, length).trim();
		if (currentNode == null || textNode.length() <= 0)
			return;
		
		if (aliasElementHit)
		{
			currentAlias += textNode;				
		}
		else if (uriElementHit)
		{
			if (elementEdge == null)
			{
				elementEdge = new ElementEdge();
				elementEdge.setSourceElement(srcReferenceNode);
			}
			elementEdge.appendToReference(textNode);
		}
	}
	
	/**
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		if (currentNode == null)
			return;
		
		if (ISMLConstants.DOCUMENT_ELEMENT.equals(qName))
		{
			String[] aliases = currentNode.getAliases();
			if (aliases == null)
				return;
			
			for (int i = 0; i < aliases.length; i++)
			{
				referenceGraph.put(aliases[i], currentNode);
			}
			
			currentNode = null;
		}
		else if (ISMLConstants.ALIAS_ELEMENT.equals(qName))
		{
			currentNode.addAlias(currentAlias);
			currentAlias = "";
		}
		else if (ISMLConstants.URI_ELEMENT.equals(qName))
		{
			if (elementEdge != null)
				currentNode.addEdge(elementEdge);
		}
	}

	
	/**
	 * The data structure returned is a Map with the following details:
	 * 
	 * KEY = ALIAS of a document
	 * VALUE = An element of type {@link DocumentNode}  
	 */
	public Object getDataStructure()
	{
		return referenceGraph;
	}
}
