/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.smlvalidators;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.artifacts.DocumentNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementEdge;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.core.AbstractSMLValidator;
import org.eclipse.cosmos.rm.validation.internal.databuilders.AcyclicDataTypesList;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.ReferenceGraphDataBuilder;
import org.eclipse.osgi.util.NLS;


/**
 * This class is used to validate the acyclic sml extension 
 * 
 * @author Ali Mehregani
 */
public class AcyclicValidator extends AbstractSMLValidator
{
	private Map nodeOrder;


	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.launcher.IValidator#intialize(java.util.Map)
	 */
	public void intialize(Map validationAttribute)
	{
		super.intialize(validationAttribute);
		
		/* Register the right data builders with the registry */
		DataBuilderRegistry databuilderRegistry = DataBuilderRegistry.instance();
		databuilderRegistry.registerDataStructureBuilder(AcyclicDataTypesList.ID, new AcyclicDataTypesList());
		databuilderRegistry.registerDataStructureBuilder(ReferenceGraphDataBuilder.ID, new ReferenceGraphDataBuilder());
	}
	

	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.launcher.IValidator#validate()
	 */
	public boolean validate()
	{
		/* Start the validation process for the acyclic extension */		
		DataBuilderRegistry databuilderRegistry = DataBuilderRegistry.instance();
		Iterator acyclicTypes = (Iterator)databuilderRegistry.getDataStructureBuilder(AcyclicDataTypesList.ID);
		Map referenceGraph = (Map)databuilderRegistry.getDataStructureBuilder(ReferenceGraphDataBuilder.ID);		
		IValidationOutput output = getValidationOutput();
		
		/* We can't continue if the reference graph is not available */
		if (referenceGraph == null || acyclicTypes == null)
		{
			/* Output error */
			output.reportError(ValidationMessages.acyclicUnknown);
			output.reportError(referenceGraph == null ? ValidationMessages.acyclicNoReferenceGraph : ValidationMessages.acyclicNoAcyclicTypeList);
			
			return false;
		}
		
		String[] graphNodes = new String[referenceGraph.size()];
		int counter = 0;
		for (Iterator iterator = referenceGraph.keySet().iterator(); iterator.hasNext();)
		{			
			graphNodes[counter++] = (String)iterator.next();			
		}
		
		/* Iterate through the acyclic element types */
		Edge backEdge = null;
		for (Iterator iter = acyclicTypes; iter.hasNext() && backEdge == null;)
		{
			String elementName = (String)iter.next();
			
			/* Because there is no guarantee that the graph is connected, we'll need to iterate
			 * through each node. */
			for (int i = 0; i < graphNodes.length && backEdge == null; i++)
			{
				if (getNodeOrder(referenceGraph, graphNodes[i], elementName) == -1)
					backEdge = dfsSearch (referenceGraph, elementName, graphNodes[i]);		
			}			
		}
		
		/* If a back edge was detected, then the reference graph is not acyclic */
		if (backEdge != null)
		{
			output.reportError(NLS.bind(ValidationMessages.acyclicCycleFound, backEdge.referenceType));
			return false;
		}
		
		return true;
	}

	
	/**
	 * Return the node order of the node that is passed in.
	 * 
	 * @param node The node order
	 * @param elementName The element name indicating the type that the reference
	 * graph should be based on
	 *  
	 * @return An integer indicating the node order
	 */
	private int getNodeOrder(Map referenceGraph, String node, String elementName)
	{
		DocumentNode documentNode = (DocumentNode)referenceGraph.get(node);
		if (documentNode == null)
			return -1;
				
		Map ordersPerElement = (Map)nodeOrder.get(elementName);
		Integer order = ordersPerElement == null ? null : (Integer)ordersPerElement.get(new Integer(documentNode.hashCode()));
		return order == null ? -1 : order.intValue();
	}

	
	/**
	 * Increment the node order of the 'node' passed in.
	 * 
	 * @param referenceGraph The reference graph
	 * @param elementName The element name represeting the type of the reference
	 * @param node  The node
	 */
	private void incrementNodeOrder (Map referenceGraph, String elementName, String node)
	{
		DocumentNode documentNode = (DocumentNode)referenceGraph.get(node);
		if (documentNode == null)
			return;
		
		Integer hashcode = new Integer(documentNode.hashCode());
		Map ordersPerElement = (Map)nodeOrder.get(elementName);
		if (ordersPerElement == null)
		{
			ordersPerElement = new Hashtable();
			nodeOrder.put(elementName, ordersPerElement);
		}
				
		Integer order = (Integer)ordersPerElement.get(hashcode);
		ordersPerElement.put(hashcode, order == null ? new Integer(0) : new Integer(order.intValue() + 1));
	}

	
	/**
	 * Perform a DFS walk through of the reference graph to determine if there
	 * are any back edges.  
	 * 
	 * @param referenceGraph The reference graph
	 * @param elementName The element name
	 * @param node The node
	 * @return The first back edge detected
	 */
	private Edge dfsSearch(Map referenceGraph, String elementName, String node)
	{
		DocumentNode documentNode = (DocumentNode)referenceGraph.get(node);
		incrementNodeOrder (referenceGraph, elementName, node);
		ElementEdge[] edges = documentNode.getEdges(elementName);
		Edge backEdge = null;
		for (int i = 0; i < edges.length; i++)
		{
			String document = edges[i].getDocument();
			if (getNodeOrder(referenceGraph, document, elementName) != -1)
			{
				backEdge = dfsSearch (referenceGraph, elementName, document);
			}
			else
			{
				backEdge = new Edge();
				backEdge.sourceDocument = node;
				backEdge.targetDocument = document;
				backEdge.referenceType = elementName;
				return backEdge;
			}
		}
		
		return backEdge;
	}
	
	
	private static class Edge
	{
		public String sourceDocument;
		public String targetDocument;
		public String referenceType;
	}
}
